#!/usr/bin/env python

import threading
import time

from dm.common.utility.loggingManager import LoggingManager
from dm.common.utility.configurationManager import ConfigurationManager
from dm.common.utility.singleton import Singleton


class StorageManager(Singleton):

    CONFIG_SECTION_NAME = 'StorageManager'
    STORAGE_DIRECTORY_KEY = 'storagedirectory'

    # Singleton.
    __instanceLock = threading.RLock()
    __instance = None

    def __init__(self):
        StorageManager.__instanceLock.acquire()
        try:
            if StorageManager.__instance:
                return
            StorageManager.__instance = self
            self.logger = LoggingManager.getInstance().getLogger(self.__class__.__name__)

            self.logger.debug('Initializing')
            self.lock = threading.RLock()
            self.__configure()
            self.logger.debug('Initialization complete')
        finally:
            StorageManager.__instanceLock.release()

    def __configure(self):
        cm = ConfigurationManager.getInstance()
        configItems = cm.getConfigItems(StorageManager.CONFIG_SECTION_NAME)
        self.logger.debug('Got config items: %s' % configItems)
        self.storageDirectory = cm.getConfigOption(StorageManager.CONFIG_SECTION_NAME, StorageManager.STORAGE_DIRECTORY_KEY)

    def createExperimentDataDirectory(self, experiment):
        self.lock.acquire()
        try:
            experimentTypeName = experiment.get('experimentType').get('rootDataPath')
            experimentName = experiment.get('name')
            dataDirectory = '%s/%s/%s' % (self.storageDirectory, experimentTypeName, experimentName)
            dataDirectory = dataDirectory.replace('//', '/')
            self.logger.debug('Creating data directory for experiment %s: %s' % (experimentName, dataDirectory))
            experiment['dataDirectory'] = dataDirectory
        finally:
            self.lock.release()
      
    def start(self):
        self.lock.acquire()
        try:
           self.logger.debug('Started storage manager')
        finally:
           self.lock.release()

    def stop(self):
        self.lock.acquire()
        try:
           self.logger.debug('Stopped storage manager')
        finally:
           self.lock.release()

####################################################################
# Testing

if __name__ == '__main__':
    sm = StorageManager.getInstance()
    print sm

